@@ -1,7 +1,9 @@
 import warnings
+from contextlib import contextmanager
 
 import pandas as pd
 
+import shapely
 import shapely.wkb
 
 from geopandas import GeoDataFrame
@@ -9,6 +11,35 @@ from geopandas import GeoDataFrame
 from .. import _compat as compat
 
 
+@contextmanager
+def _get_conn(conn_or_engine):
+    """
+    Yield a connection within a transaction context.
+
+    Engine.begin() returns a Connection with an implicit Transaction while
+    Connection.begin() returns the Transaction. This helper will always return a
+    Connection with an implicit (possibly nested) Transaction.
+
+    Parameters
+    ----------
+    conn_or_engine : Connection or Engine
+        A sqlalchemy Connection or Engine instance
+    Returns
+    -------
+    Connection
+    """
+    from sqlalchemy.engine.base import Engine, Connection
+
+    if isinstance(conn_or_engine, Connection):
+        with conn_or_engine.begin():
+            yield conn_or_engine
+    elif isinstance(conn_or_engine, Engine):
+        with conn_or_engine.begin() as conn:
+            yield conn
+    else:
+        raise ValueError(f"Unknown Connectable: {conn_or_engine}")
+
+
 def _df_to_geodf(df, geom_col="geom", crs=None):
     """
     Transforms a pandas DataFrame into a GeoDataFrame.
@@ -55,7 +86,10 @@ def _df_to_geodf(df, geom_col="geom", crs=None):
 
         df[geom_col] = geoms = geoms.apply(load_geom)
         if crs is None:
-            srid = shapely.geos.lgeos.GEOSGetSRID(geoms.iat[0]._geom)
+            if compat.SHAPELY_GE_20:
+                srid = shapely.get_srid(geoms.iat[0])
+            else:
+                srid = shapely.geos.lgeos.GEOSGetSRID(geoms.iat[0]._geom)
             # if no defined SRID in geodatabase, returns SRID of 0
             if srid != 0:
                 crs = "epsg:{}".format(srid)
@@ -83,7 +117,7 @@ def _read_postgis(
     sql : string
         SQL query to execute in selecting entries from database, or name
         of the table to read from the database.
-    con : DB connection object or SQLAlchemy engine
+    con : sqlalchemy.engine.Connection or sqlalchemy.engine.Engine
         Active connection to the database to query.
     geom_col : string, default 'geom'
         column name to convert to shapely geometries
@@ -106,10 +140,17 @@ def _read_postgis(
     Examples
     --------
     PostGIS
-    >>> sql = "SELECT geom, kind FROM polygons"
+
+    >>> from sqlalchemy import create_engine  # doctest: +SKIP
+    >>> db_connection_url = "postgresql://myusername:mypassword@myhost:5432/mydatabase"
+    >>> con = create_engine(db_connection_url)  # doctest: +SKIP
+    >>> sql = "SELECT geom, highway FROM roads"
+    >>> df = geopandas.read_postgis(sql, con)  # doctest: +SKIP
+
     SpatiaLite
-    >>> sql = "SELECT ST_AsBinary(geom) AS geom, kind FROM polygons"
-    >>> df = geopandas.read_postgis(sql, con)
+
+    >>> sql = "SELECT ST_Binary(geom) AS geom, highway FROM roads"
+    >>> df = geopandas.read_postgis(sql, con)  # doctest: +SKIP
     """
 
     if chunksize is None:
@@ -145,7 +186,7 @@ def read_postgis(*args, **kwargs):
     warnings.warn(
         "geopandas.io.sql.read_postgis() is intended for internal "
         "use only, and will be deprecated. Use geopandas.read_postgis() instead.",
-        DeprecationWarning,
+        FutureWarning,
         stacklevel=2,
     )
 
@@ -170,7 +211,7 @@ def _get_geometry_type(gdf):
           such as GeometryCollection([Point, LineStrings])
      - if any of the geometries has Z-coordinate, all records will
        be written with 3D.
-     """
+    """
     geom_types = list(gdf.geometry.geom_type.unique())
     has_curve = False
 
@@ -193,7 +234,7 @@ def _get_geometry_type(gdf):
 
     # Check for 3D-coordinates
     if any(gdf.geometry.has_z):
-        target_geom_type = target_geom_type + "Z"
+        target_geom_type += "Z"
 
     return target_geom_type, has_curve
 
@@ -205,19 +246,30 @@ def _get_srid_from_crs(gdf):
 
     # Use geoalchemy2 default for srid
     # Note: undefined srid in PostGIS is 0
-    srid = -1
+    srid = None
     warning_msg = (
         "Could not parse CRS from the GeoDataFrame. "
-        + "Inserting data without defined CRS.",
+        "Inserting data without defined CRS."
     )
     if gdf.crs is not None:
         try:
-            srid = gdf.crs.to_epsg(min_confidence=25)
-            if srid is None:
-                srid = -1
-                warnings.warn(warning_msg, UserWarning, stacklevel=2)
+            for confidence in (100, 70, 25):
+                srid = gdf.crs.to_epsg(min_confidence=confidence)
+                if srid is not None:
+                    break
+                auth_srid = gdf.crs.to_authority(
+                    auth_name="ESRI", min_confidence=confidence
+                )
+                if auth_srid is not None:
+                    srid = int(auth_srid[1])
+                    break
         except Exception:
             warnings.warn(warning_msg, UserWarning, stacklevel=2)
+
+    if srid is None:
+        srid = -1
+        warnings.warn(warning_msg, UserWarning, stacklevel=2)
+
     return srid
 
 
@@ -235,8 +287,15 @@ def _convert_linearring_to_linestring(gdf, geom_name):
 
 
 def _convert_to_ewkb(gdf, geom_name, srid):
-    """Convert geometries to ewkb. """
-    if compat.USE_PYGEOS:
+    """Convert geometries to ewkb."""
+    if compat.USE_SHAPELY_20:
+        geoms = shapely.to_wkb(
+            shapely.set_srid(gdf[geom_name].values.data, srid=srid),
+            hex=True,
+            include_srid=True,
+        )
+
+    elif compat.USE_PYGEOS:
         from pygeos import set_srid, to_wkb
 
         geoms = to_wkb(
@@ -269,7 +328,9 @@ def _psql_insert_copy(tbl, conn, keys, data_iter):
 
     dbapi_conn = conn.connection
     with dbapi_conn.cursor() as cur:
-        sql = "COPY {} ({}) FROM STDIN WITH CSV".format(tbl.table.fullname, columns)
+        sql = 'COPY "{}"."{}" ({}) FROM STDIN WITH CSV'.format(
+            tbl.table.schema, tbl.table.name, columns
+        )
         cur.copy_expert(sql=sql, file=s_buf)
 
 
@@ -294,7 +355,7 @@ def _write_postgis(
     ----------
     name : str
         Name of the target table.
-    con : sqlalchemy.engine.Engine
+    con : sqlalchemy.engine.Connection or sqlalchemy.engine.Engine
         Active connection to the PostGIS database.
     if_exists : {'fail', 'replace', 'append'}, default 'fail'
         How to behave if the table already exists:
@@ -322,23 +383,15 @@ def _write_postgis(
     Examples
     --------
 
-    >>> from sqlalchemy import create_engine
-    >>> engine = create_engine("postgres://myusername:mypassword@myhost:5432\
-/mydatabase";)
-    >>> gdf.to_postgis("my_table", engine)
+    >>> from sqlalchemy import create_engine  # doctest: +SKIP
+    >>> engine = create_engine("postgresql://myusername:mypassword@myhost:5432\
+/mydatabase";)  # doctest: +SKIP
+    >>> gdf.to_postgis("my_table", engine)  # doctest: +SKIP
     """
     try:
         from geoalchemy2 import Geometry
     except ImportError:
-        raise ImportError("'to_postgis()' requires geoalchemy2 package. ")
-
-    if not compat.SHAPELY_GE_17:
-        raise ImportError(
-            "'to_postgis()' requires newer version of Shapely "
-            "(>= '1.7.0').\nYou can update the library using "
-            "'pip install shapely --upgrade' or using "
-            "'conda update shapely' if using conda package manager."
-        )
+        raise ImportError("'to_postgis()' requires geoalchemy2 package.")
 
     gdf = gdf.copy()
     geom_name = gdf.geometry.name
@@ -362,16 +415,16 @@ def _write_postgis(
     # Convert geometries to EWKB
     gdf = _convert_to_ewkb(gdf, geom_name, srid)
 
+    if schema is not None:
+        schema_name = schema
+    else:
+        schema_name = "public"
+
     if if_exists == "append":
         # Check that the geometry srid matches with the current GeoDataFrame
-        with con.begin() as connection:
-            if schema is not None:
-                schema_name = schema
-            else:
-                schema_name = "public"
-
+        with _get_conn(con) as connection:
             # Only check SRID if table exists
-            if connection.run_callable(connection.dialect.has_table, name, schema):
+            if connection.dialect.has_table(connection, name, schema):
                 target_srid = connection.execute(
                     "SELECT Find_SRID('{schema}', '{table}', '{geom_col}');".format(
                         schema=schema_name, table=name, geom_col=geom_name
@@ -387,12 +440,12 @@ def _write_postgis(
                     )
                     raise ValueError(msg)
 
-    with con.begin() as connection:
+    with _get_conn(con) as connection:
 
         gdf.to_sql(
             name,
             connection,
-            schema=schema,
+            schema=schema_name,
             if_exists=if_exists,
             index=index,
             index_label=index_label,
