@@ -37,9 +37,9 @@ def test_get_deps_info():
     assert "pyproj" in deps_info
     assert "matplotlib" in deps_info
     assert "mapclassify" in deps_info
-    assert "pysal" in deps_info
     assert "geopy" in deps_info
     assert "psycopg2" in deps_info
+    assert "geoalchemy2" in deps_info
 
 
 def test_show_versions(capsys):
