@@ -1,4 +1,3 @@
-from .crs import explicit_crs_from_epsg
 from .geocoding import geocode, reverse_geocode
 from .overlay import overlay
 from .sjoin import sjoin, sjoin_nearest
@@ -7,7 +6,6 @@ from .clip import clip
 
 __all__ = [
     "collect",
-    "explicit_crs_from_epsg",
     "geocode",
     "overlay",
     "reverse_geocode",
